package player;

import edu.neu.ccs.demeterf.lib.List;
import scg.gen.PlayerContext;
import logging.Logger;
import scg.gen.*;

public class PlayerFactory {
    static final String
         INFINITE = "--infiniteloop",
         NOOFFERS = "--nooffers",
         EMPTYTRANS = "--emptytrans";
    
    static List<String> validOptions(){ return List.create(INFINITE,NOOFFERS); }
    
    List<String> options;
    
    private PlayerFactory(List<String> opts){ options = opts; }
    
    /** Create a PlayerFactory with the given command line options */
    public static PlayerFactory create(List<String> opts){ return new PlayerFactory(opts); }
    
    /** Crate a Player with teh given command line Options, context, and logger */
    public Player getAPlayer(PlayerContext ctxt, Logger l){
        if(options.contains(INFINITE))return new InfiniteLoop(ctxt, l);
        if(options.contains(NOOFFERS))return new NoOffers(ctxt, l);
        if(options.contains(EMPTYTRANS))return new EmptyTrans(ctxt, l);
        return Player.create(ctxt, l);
    }
    
    
    /* Different, Unruly Players... */
    
    /** Infinite loop */
    static class InfiniteLoop extends Player{
        InfiniteLoop(PlayerContext ctxt, Logger l){ super(ctxt,l); }
        public PlayerTrans play(){
            if(Math.random() > .5)return super.play();
            while(true)
                try{
                    log.notify("Still Waiting");
                    Thread.sleep(10000);
                }catch(InterruptedException e){}
        }
    }
    
    /** Empty Transaction */
    static class EmptyTrans extends Player{
        EmptyTrans(PlayerContext ctxt, Logger l){ super(ctxt,l); }
        public PlayerTrans play(){
            if(Math.random() > .5)return super.play();
            return new PlayerTrans(context.getId(), List.<Transaction>create());
        }
    }
    
    /** No Offered Transactions */
    static class NoOffers extends Player{
        NoOffers(PlayerContext ctxt, Logger l){ super(ctxt,l); }
        public PlayerTrans play(){
            if(Math.random() > .5)return super.play();
            return new PlayerTrans(context.getId(),
                    super.play().getTs().filterout(new List.Pred<Transaction>(){
                        public boolean huh(Transaction t){ return t instanceof OfferTrans; } 
                    }));
        }
    }
}
