package test;

import java.util.Date;

import logging.Logger;
import player.PlayerFactory;
import player.PlayerServer;
import scg.Constants;
import scg.Util;
import scg.game.HistoryFile;
import scg.game.LocalPlayerProxy;
import scg.game.PlayerProxy;
import scg.game.Tournament;
import scg.gen.Config;
import scg.gen.PlayerSpec;
import edu.neu.ccs.demeterf.lib.List;
import java.io.FileInputStream;

/** Simple setup for a local (single machine) run without using network sockets */
public class LocalGameTest {

    public static void main(final String[] args) throws Exception{
        // 0) A bit of configuration
        final int numPlayers = 3;
        PlayerFactory.setGlobalOptions(List.create(args).append(PlayerFactory.NOSECRETS));

        // 1) Create a configuration
        Config config =
        // By Hand...
        // new Config("CSP", 60, 0.01, initialAccnt, 5, 1, 1, new Objective(),
        // new TBall(1000), numRounds, 1.0, 2, true, 0.2);
        // Or from a file...
        // Config.parse(new java.io.FileInputStream(Constants.CONFIG_FILE));
        // Or from a String...
        Config.parse(new FileInputStream("files/config.txt"));

        // 2) Create some players
        final Logger logger = Logger.text(System.out);
        List<PlayerProxy> players = List.buildlist(new List.Build<PlayerProxy>() {

            @Override
            public PlayerProxy build(int i){
                return new LocalPlayerProxy(new PlayerSpec("Player" + i, "", 0), new PlayerServer(PlayerFactory
                        .create(List.create(args).append(PlayerFactory.SMART)), logger));
            }
        }, numPlayers);

        // 3) Create a History file
        Date start = Util.now();
        HistoryFile history = new HistoryFile("history", start, Constants.HISTORY_FILE_SUFFIX);
        // 4) Create a Game instance
        // Game game = new Game(config, players.toJavaList());
        // Or a Tournament instance ...
        Tournament game = new Tournament(config, players.toJavaList());

        // 5) Run the competition
        game.start(history);
        // Or
        // StatisticsCollector sc = new StatisticsCollector("statistics", start,
        // Constants.HISTORY_FILE_SUFFIX);
        // game.start(history, sc);

    }
}
