package scg.hidden;

import scg.gen.Clause;
import scg.gen.Problem;
import scg.gen.Solution;
import scg.gen.Var;
import edu.neu.ccs.demeterf.lib.Map;
import edu.neu.ccs.evergreen.ir.RelationCore;

/** Helpful functions that cannot (yet) be made public */
public class CSP {

    /** Calculate the value/quality of a Solution */
    public static double value(Problem inst, Solution sol){
        Map<Var,Boolean> assign = Map.create(sol.getAssign().toList());
        double satWeight = 0;
        double totalWeight = 0;
        for (Clause clause : inst.getClauses()) {
            if (isSat(clause, assign)) {
                satWeight += clause.getWeight();
            }
            totalWeight += clause.getWeight();
        }
        return totalWeight > 0 ? (satWeight / totalWeight) : 1.0;
    }

    private static boolean isSat(Clause clause, Map<Var,Boolean> sol){
        int rno = clause.getRelnum();
        int i = 0;
        for (Var var : clause.getVars()) {
            if (sol.containsKey(var)) {
                Boolean val = sol.get(var);
                rno = RelationCore.reduce(rno, 3, i, val ? 1 : 0);
            } else {
                System.err.println("!!INCOMPLETE ASSIGNMENT!!");
            }
            i++;
            if (rno == 255) {
                return true;
            }
            if (rno == 0) {
                return false;
            }
        }
        return rno == 255;
    }
}